import math;
import hyperbolic_geometry;

size(500,500);
settings.outformat="pdf";

// Here we show the construction of the incircle of a triangle.
// This is done in analogy to the classical euclidean geometry.

hyperbolic_point O = hyperbolic_point(0,0);
dot(O,red);

// The two points A, B given
// hyperbolic_point A = hyperbolic_point(1.0,60);
// hyperbolic_point B = hyperbolic_point(1.0,180);
// hyperbolic_point C = hyperbolic_point(1.0,300);

// Triangle without incircle:
hyperbolic_point A = hyperbolic_point(0.5,330);
hyperbolic_point B = hyperbolic_point(2.4,60);
hyperbolic_point C = hyperbolic_point(1.1,180);

// The triangle
hyperbolic_triangle T = hyperbolic_triangle(A,B,C);

// The sides
hyperbolic_line AB = hyperbolic_line(A,B);
hyperbolic_line BC = hyperbolic_line(B,C);
hyperbolic_line AC = hyperbolic_line(A,C);

//------------------------------------------------------------------------------
// Construction angle bisection of alpha

// Construct equidistant point on both lines
hyperbolic_circle c = hyperbolic_circle(A,O);
hyperbolic_point[] K = intersection(c,AB);
hyperbolic_point[] L = intersection(c,AC);

// Construct the intersecting circles
hyperbolic_circle c1 = hyperbolic_circle(K[0], A);
hyperbolic_circle c2 = hyperbolic_circle(L[1], A);

// The intersection of the two circles
hyperbolic_point[] M = intersection(c1, c2);

// Constrcut the bisecting line
hyperbolic_line WA = hyperbolic_line(M[0], M[1]);

hyperbolic_point IBC = intersection(BC, WA);

draw(c, gray+opacity(0.3));
draw(c1, gray+opacity(0.3));
draw(c2, gray+opacity(0.3));
dot(M, gray);

//------------------------------------------------------------------------------
// Construction angle bisection of beta

// Construct equidistant point on both lines
hyperbolic_circle c = hyperbolic_circle(B,O);
hyperbolic_point[] K = intersection(c,AB);
hyperbolic_point[] L = intersection(c,BC);

// Construct the intersecting circles
hyperbolic_circle c1 = hyperbolic_circle(K[1], B);
hyperbolic_circle c2 = hyperbolic_circle(L[0], B);

// The intersection of the two circles
hyperbolic_point[] M = intersection(c1, c2);

// Constrcut the bisecting line
hyperbolic_line WB = hyperbolic_line(M[0], M[1]);

hyperbolic_point IAC = intersection(AC, WB);

draw(c, gray+opacity(0.3));
draw(c1, gray+opacity(0.3));
draw(c2, gray+opacity(0.3));
dot(M, gray);

//------------------------------------------------------------------------------
// Construction angle bisection of gamma

// Construct equidistant point on both lines
hyperbolic_circle c = hyperbolic_circle(C,O);
hyperbolic_point[] K = intersection(c,AC);
hyperbolic_point[] L = intersection(c,BC);

// Construct the intersecting circles
hyperbolic_circle c1 = hyperbolic_circle(K[0], C);
hyperbolic_circle c2 = hyperbolic_circle(L[1], C);

// The intersection of the two circles
hyperbolic_point[] M = intersection(c1, c2);

// Constrcut the bisecting line
hyperbolic_line WC = hyperbolic_line(M[0], M[1]);

hyperbolic_point IAB = intersection(AB, WC);

draw(c, gray+opacity(0.3));
draw(c1, gray+opacity(0.3));
draw(c2, gray+opacity(0.3));
dot(M, gray);

// And calculate the center of the incircle
hyperbolic_point ICC = intersection(WA, WB);

// Now get the perpendicular to one side
hyperbolic_line N = hyperbolic_normal(AB, ICC);

// And get a point of the incircle
hyperbolic_point X = intersection(N, T.AB());

// And the incircle itself
hyperbolic_circle INCIR = hyperbolic_circle(ICC, X);


// Draw
fill(T, blue+opacity(0.3));
draw(T);
dot(T);
draw(WA, blue+opacity(0.3));
draw(WB, blue+opacity(0.3));
draw(WC, blue+opacity(0.3));
draw(N, yellow);
draw(INCIR, orange);
dot(IAB, blue);
dot(IBC, blue);
dot(IAC, blue);
dot(X, yellow);
dot(ICC, orange);
dot(O, red);


draw((1,1)--(1,-1)--(-1,-1)--(-1,1)--cycle);
draw(unitcircle);
